@@ -183,6 +183,7 @@
 #define HAVE_MLOCKALL 1
 #define HAVE_MMAP 1
 /* #undef HAVE_MMAP64 */
+#define HAVE_MPROTECT 1
 #define HAVE_PERROR 1
 #define HAVE_POLL 1
 /* #undef HAVE_POSIX_FALLOCATE */
@@ -398,8 +399,8 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "OpenBSD"
-#define MACHINE_TYPE "amd64"
-#define DEFAULT_MACHINE "amd64"
+#define MACHINE_TYPE "riscv64"
+#define DEFAULT_MACHINE "riscv64"
 /* #undef HAVE_DTRACE */
 
 #define SIGNAL_WITH_VIO_CLOSE 1
@@ -533,17 +534,17 @@
 /* #undef SO_EXT */
 
 #define MYSQL_VERSION_MAJOR 10
-#define MYSQL_VERSION_MINOR 6
-#define MYSQL_VERSION_PATCH 10
+#define MYSQL_VERSION_MINOR 9
+#define MYSQL_VERSION_PATCH 3
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.6.10"
+#define PACKAGE_STRING "MySQL Server 10.9.3"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.6.10"
-#define VERSION "10.6.10"
+#define PACKAGE_VERSION "10.9.3"
+#define VERSION "10.9.3"
 #define PROTOCOL_VERSION 10
 #define PCRE2_CODE_UNIT_WIDTH 8
 
