@@ -134,7 +134,7 @@
 #define OPTION_BEGIN            (1ULL << 20)    // THD, intern
 #define OPTION_TABLE_LOCK       (1ULL << 21)    // THD, intern
 #define OPTION_QUICK            (1ULL << 22)    // SELECT (for DELETE)
-#define OPTION_KEEP_LOG         (1ULL << 23)    // THD, user
+#define OPTION_BINLOG_THIS_TRX  (1ULL << 23)    // THD
 
 #define OPTION_EXPLICIT_DEF_TIMESTAMP   (1ULL << 24) // THD, user
 #define OPTION_GTID_BEGIN       (1ULL << 25)    // GTID BEGIN found in log
@@ -174,6 +174,9 @@
 */
 #define OPTION_MASTER_SQL_ERROR         (1ULL << 35)
 
+#define OPTION_BINLOG_THIS_STMT         (1ULL << 36) // THD
+#define OPTION_BINLOG_THIS (OPTION_BINLOG_THIS_STMT | OPTION_BINLOG_THIS_TRX)
+
 #define OPTION_SKIP_REPLICATION         (1ULL << 37) // THD, user
 #define OPTION_RPL_SKIP_PARALLEL        (1ULL << 38)
 #define OPTION_NO_QUERY_CACHE           (1ULL << 39) // SELECT, user
@@ -334,6 +337,8 @@
 #define UNCACHEABLE_DEPENDENT (UNCACHEABLE_DEPENDENT_GENERATED | \
                                UNCACHEABLE_DEPENDENT_INJECTED)
 
+#define FAKE_SELECT_LEX_ID UINT_MAX
+
 /* Used to check GROUP BY list in the MODE_ONLY_FULL_GROUP_BY mode */
 #define UNDEF_POS (-1)
 
@@ -416,16 +421,6 @@ inline int hexchar_to_int(char c)
 /* This must match the path length limit in the ER_NOT_RW_DIR error msg. */
 #define ER_NOT_RW_DIR_PATHSIZE 200
 
-#define IS_TABLESPACES_TABLESPACE_NAME    0
-#define IS_TABLESPACES_ENGINE             1
-#define IS_TABLESPACES_TABLESPACE_TYPE    2
-#define IS_TABLESPACES_LOGFILE_GROUP_NAME 3
-#define IS_TABLESPACES_EXTENT_SIZE        4
-#define IS_TABLESPACES_AUTOEXTEND_SIZE    5
-#define IS_TABLESPACES_MAXIMUM_SIZE       6
-#define IS_TABLESPACES_NODEGROUP_ID       7
-#define IS_TABLESPACES_TABLESPACE_COMMENT 8
-
 bool db_name_is_in_ignore_db_dirs_list(const char *dbase);
 
 #endif /* MYSQL_SERVER */
