@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Codership Oy <info@codership.com>
+/* Copyright (C) 2013-2021 Codership Oy <info@codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@ class set_var;
 class THD;
 
 int wsrep_init_vars();
+void wsrep_set_wsrep_on(THD *thd);
 
 #define CHECK_ARGS   (sys_var *self, THD* thd, set_var *var)
 #define UPDATE_ARGS  (sys_var *self, THD* thd, enum_var_type type)
@@ -106,6 +107,8 @@ extern bool wsrep_debug_update               UPDATE_ARGS;
 extern bool wsrep_gtid_seq_no_check          CHECK_ARGS;
 
 extern bool wsrep_gtid_domain_id_update      UPDATE_ARGS;
+
+extern bool wsrep_mode_check                 CHECK_ARGS;
 #else  /* WITH_WSREP */
 
 #define wsrep_provider_init(X)
